import base64
import json
import logging

import boto3
from botocore.exceptions import ClientError

_LOGGER = logging.getLogger(__name__)

DEFAULT_REGION_NAME = "eu-west-1"


def get_secret_parser(secret_name, region_name=DEFAULT_REGION_NAME):
    try:
        secret = get_secret(secret_name, region_name)
    except botocore.exceptions.NoCredentialsError:
        _LOGGER.error("No credentials")
        return {}
    if secret is None:
        return {}
    return json.loads(secret)


def get_secret(secret_name, region_name=DEFAULT_REGION_NAME):

    # Create a Secrets Manager client
    session = boto3.session.Session()
    client = session.client(service_name="secretsmanager", region_name=region_name)

    try:
        get_secret_value_response = client.get_secret_value(SecretId=secret_name)
    except ClientError as e:
        if e.response["Error"]["Code"] == "DecryptionFailureException":
            # Secrets Manager can't decrypt the protected secret text using the provided KMS key.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response["Error"]["Code"] == "InternalServiceErrorException":
            # An error occurred on the server side.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response["Error"]["Code"] == "InvalidParameterException":
            # You provided an invalid value for a parameter.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response["Error"]["Code"] == "InvalidRequestException":
            # You provided a parameter value that is not valid for the current state of the resource.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response["Error"]["Code"] == "ResourceNotFoundException":
            # We can't find the resource that you asked for.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
    else:
        # Decrypts secret using the associated KMS CMK.
        # Depending on whether the secret is a string or binary, one of these fields will be populated.
        if "SecretString" in get_secret_value_response:
            return get_secret_value_response["SecretString"]
        return base64.b64decode(get_secret_value_response["SecretBinary"])



import json
import logging
import time

import aiobotocore

_LOGGER = logging.getLogger(__name__)


class Queue:
    def __init__(self, queue_name, region_name="eu-west-1"):
        self._queue_name = queue_name
        self._region_name = region_name
        self._session = aiobotocore.get_session()
        self._client = self._session.create_client("sqs", region_name=region_name)
        self.queue_url = None

    async def subscribe_topic(self, topic_name):

        response = await self._client.create_queue(QueueName=self._queue_name)
        self.queue_url = response["QueueUrl"]
        attr_response = await self._client.get_queue_attributes(
            QueueUrl=self.queue_url, AttributeNames=["All"]
        )

        queue_attributes = attr_response.get("Attributes")
        queue_arn = queue_attributes.get("QueueArn")

        # Set up a policy to allow SNS access to the queue
        if "Policy" in queue_attributes:
            policy = json.loads(queue_attributes["Policy"])
        else:
            policy = {
                "Version": "2012-10-17",
                "Statement": [
                    {
                        "Sid": "Sid" + str(int(time.time())),
                        "Effect": "Allow",
                        "Principal": {"AWS": "*"},
                        "Action": ["sqs:SendMessage", "sqs:ReceiveMessage"],
                    }
                ],
            }

        statement = policy.get("Statement", [{}])[0]
        statement["Resource"] = statement.get("Resource", queue_arn)
        statement["Condition"] = statement.get("Condition", {})

        statement["Condition"]["StringLike"] = statement["Condition"].get(
            "StringLike", {}
        )
        source_arn = statement["Condition"]["StringLike"].get("aws:SourceArn", [])
        if not isinstance(source_arn, list):
            source_arn = [source_arn]

        sns = self._session.create_client("sns", region_name=self._region_name)
        response = await sns.create_topic(Name=topic_name)
        topic_arn = response["TopicArn"]

        if topic_arn not in source_arn:
            source_arn.append(topic_arn)
            statement["Condition"]["StringLike"]["aws:SourceArn"] = source_arn
            policy["Statement"] = statement
            await self._client.set_queue_attributes(
                QueueUrl=self.queue_url, Attributes={"Policy": json.dumps(policy)}
            )

        await sns.subscribe(TopicArn=topic_arn, Protocol="sqs", Endpoint=queue_arn)
        await sns.close()

    async def send(self, subject, message, delay_seconds=0):
        if self.queue_url is None:
            _LOGGER.error("No subscribed queue")
            return None

        return await self.queue.send_message(
            QueueUrl=self.queue.url,
            DelaySeconds=delay_seconds,
            MessageBody=json.dumps(
                {"Subject": subject, "Message": json.dumps(message)}
            ),
        )

    async def receive_message(self, num_msgs=1):
        if self.queue_url is None:
            _LOGGER.error("No subscribed queue")
            return [None]
        response = await self._client.receive_message(
            QueueUrl=self.queue_url, MaxNumberOfMessages=num_msgs
        )
        res = []
        for msg in response.get("Messages", []):
            res.append(MessageHandle(msg))
        return res

    async def delete_message(self, msg_handle):
        await self._client.delete_message(
            QueueUrl=self.queue_url, ReceiptHandle=msg_handle.receipt_handle
        )

    async def close(self):
        await self._client.close()


class MessageHandle:
    def __init__(self, msg):
        self._msg = msg

    @property
    def body(self):
        return self._msg["Body"]

    @property
    def receipt_handle(self):
        return self._msg["ReceiptHandle"]



import asyncio
import json
import logging
import numpy as np
import os
from urllib.parse import urlparse

import aiohttp
import async_timeout
from botocore.auth import SigV4Auth
from botocore.awsrequest import AWSRequest
from botocore.session import Session

_LOGGER = logging.getLogger(__name__)

CREDS = Session().get_credentials()
LAMBDA_ENDPOINT_BASE = "https://lambda.eu-west-1.amazonaws.com/2015-03-31/functions"
LAMBDA_TIMEOUT = 120


async def invoke(func_name, payload, aiohttp_session, retries=3):
    """Used to invoke lambda functions async."""

    def convert(o):
        if isinstance(o, np.int64):
            _LOGGER.debug("Int63, %s %s", o, payload)
            return int(0)
        if isinstance(o, dict):
            return {k: convert(v) for k, v in o.items()}
        if isinstance(o, (list, tuple)):
            return [convert(x) for x in o]
        return o

    def create_signed_headers(_url, _payload):
        host_segments = urlparse(_url).netloc.split(".")
        service = host_segments[0]
        region = host_segments[1]
        try:
            data = json.dumps(convert(_payload))
        except TypeError:
            _LOGGER.error("Failed to convert to json, %s", _payload, exc_info=True)
            raise
        request = AWSRequest(method="POST", url=_url, data=data)
        SigV4Auth(CREDS, service, region).add_auth(request)
        return dict(request.headers.items())

    url = os.path.join(LAMBDA_ENDPOINT_BASE, func_name, "invocations")
    signed_headers = create_signed_headers(url, payload)

    def log(msg, retry):
        if retry > 1:
            _LOGGER.warning(msg)
            return
        _LOGGER.error(msg)

    for retry in range(retries, 0, -1):
        try:
            with async_timeout.timeout(LAMBDA_TIMEOUT):
                try:
                    async with aiohttp_session.post(
                        url, json=payload, headers=signed_headers
                    ) as response:
                        if response.status != 200:
                            msg = await response.json()
                            log(
                                f"Error getting data from {func_name}, resp code: {response.status}, {msg}",
                                retry,
                            )
                            continue
                        return await response.json()
                except aiohttp.client_exceptions.ClientConnectorError:
                    log("ClientConnectorError", retry)
                    continue
        except asyncio.TimeoutError:
            log("Timed out", retry)
            continue

    _LOGGER.error("Error getting data from %s", func_name)
    return {}



from setuptools import setup

setup(
    name='tibber_aws',
    packages=['tibber_aws'],
    install_requires=['boto3', 'aiobotocore==0.12.0', 'aiohttp', 'async_timeout', 'numpy'],
    version='0.3.2',
    description='A python3 library to communicate with Aws',
    python_requires='>=3.5.3',
    author='Tibber',
    author_email='hello@tibber.com',
    url='https://github.com/tibbercom/tibber-pyAws',
    license="MIT",
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules'
        ]
)



import datetime
import logging
import zlib

import aiobotocore
import botocore

_LOGGER = logging.getLogger(__name__)

STATE_NOT_EXISTING = "not_existing"
STATE_OK = "ok"
STATE_PRECONDITION_FAILED = "precondition_failed"


class S3Bucket:
    def __init__(self, bucket_name, region_name="eu-west-1"):
        self._bucket_name = bucket_name
        self._region_name = region_name
        session = aiobotocore.get_session()
        self.client = session.create_client("s3", region_name=region_name)

    async def load_data(self, key, if_unmodified_since=None):
        if if_unmodified_since is None:
            if_unmodified_since = datetime.datetime(1900, 1, 1)
        try:
            res = await (
                await self.client.get_object(
                    Bucket=self._bucket_name,
                    Key=key,
                    IfUnmodifiedSince=if_unmodified_since,
                )
            )["Body"].read()
        except self.client.exceptions.NoSuchKey:
            return None, STATE_NOT_EXISTING
        except botocore.exceptions.ClientError as exp:
            if "PreconditionFailed" in str(exp):
                return None, STATE_PRECONDITION_FAILED
            raise
        if len(key) > 3 and key[-3:] == ".gz":
            content = zlib.decompressobj(zlib.MAX_WBITS | 16).decompress(res)
            return content, STATE_OK
        return res.decode("utf-8"), STATE_OK

    async def store_data(self, key, data):
        if len(key) > 3 and key[-3:] == ".gz":
            compressor = zlib.compressobj(wbits=zlib.MAX_WBITS | 16)
            body = compressor.compress(data)
            body += compressor.flush()
        else:
            body = data
        try:
            resp = await self.client.put_object(
                Bucket=self._bucket_name, Key=key, Body=body
            )
        except self.client.exceptions.NoSuchBucket:
            await self.client.create_bucket(Bucket=self._bucket_name,
                                            CreateBucketConfiguration={'LocationConstraint': self._region_name})
            return await self.store_data(key, data)
        return resp

    async def close(self):
        await self.client.close()
